/*
  ResHiggs code based on

    M. Bonvini, S. Marzani
    arXiv:1405.3654

  please cite it if you use it
*/

#include <iomanip>
#include <vector>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdio.h>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <sys/stat.h> 
#include <sys/time.h> 
#include <getopt.h> 

#include "../ggHiggs/src/ggHiggs.hh"
#include "./src/resummationQCD.hh"

#include "LHAPDF/LHAPDF.h"


using namespace std;
using namespace ggHiggs;



/*
  If you like, you can customize your alpha_s routine.
  However, consistency with PDF sets suggests the use of the LHAPDF alphas routine.
*/
double resumm::alphas(double mu) {
  return LHAPDF::alphasPDF(mu);
  //return alphas_ihixs(mu, order, LHAPDF::alphasPDF(mZ), mZ); // taken from ihixs code
}
double ggHiggs::alphas(double mu) {
  return resumm::alphas(mu);
}





void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -p  --pdf-set <set>     specify PDFs being used (default \"NNPDF23_nnlo_as_0118\")" << endl
       << " -m  --mass <mass>       set Higgs mass [GeV]" << endl
       << " -E  --Ecm <energy>      set collider energy [GeV]" << endl
       << " -f  --fact <murat>      set muF/mH ratio" << endl
       << " -r  --ren <murat>       set muR/mH ratio" << endl
       << " -t  --topmass <mass>    set top quark mass [GeV]" << endl
       << " -b  --include-bottom    include the bottom Yukawa coupling" << endl
       << " -c  --include-charm     include the charm  Yukawa coupling" << endl
       << " -e  --noEFT             use finite quark masses (default: infinite-mtop EFT approximation)" << endl
       << " -P  --prescription <p>  set resummation prescription: A-soft_2 (0), psi-soft_2 (1, default), N-soft (2)" << endl
       << " -g  --nog0exp           do NOT exponentiate constant function g0 or g0bar" << endl
       << endl;
  exit(0);
  return;
}




double muRrat = 1;
double muFrat = 1;
bool g0exp = true;
int prescr = 1;

const string sorder[4] = {"LO", "NLO", "NNLO", "NNNLO"};
void read_arguments(int argc, char* argv[], double &mH, double &sqrts, string &set, bool &nob, bool &noc) {
  const char* const short_options = "ht:ep:f:r:m:E:P:gbc";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "topmass", 1, NULL, 't' },
					 { "noEFT", 0, NULL, 'e' },
					 { "pdf-set", 1, NULL, 'p' },
					 { "fact", 1, NULL, 'f' },
					 { "ren", 1, NULL, 'r' },
					 { "mass", 1, NULL, 'm' },
					 { "Ecm", 1, NULL, 'E' },
					 { "prescription", 1, NULL, 'P' },
					 { "g0exp", 0, NULL, 'g' },
					 { "include-bottom", 0, NULL, 'b' },
					 { "include-charm", 0, NULL, 'c' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  ostringstream sset;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 't':
      mtop = strtod(optarg, NULL);
      break;
    case 'e':
      finite_mt = true;
      break;
    case 'p':
      sset << optarg;
      set = sset.str();
      break;
    case 'f':
      muFrat = strtod(optarg, NULL);
      break;
    case 'r':
      muRrat = strtod(optarg, NULL);
      break;
    case 'm':
      mH = strtod(optarg, NULL);
      break;
    case 'E':
      sqrts = strtod(optarg, NULL);
      break;
    case 'P':
      prescr = strtol(optarg, NULL, 10);
      break;
    case 'g':
      g0exp = false;
      break;
    case 'b':
      nob = false;
      break;
    case 'c':
      noc = false;
      break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  //
  // messages
  //
  if(prescr==2 && g0exp==true)
    cout << endl << "\033[1m  WARNING:\033[0m "
	 << "\033[0;31m Using N-soft with g0 exponentiated: this is NOT the standard N-soft resummation prescription! Consider using -g option.\033[0m"
	 << endl << endl;
  //
  _muF_mu_ratio  = muFrat;
  _muR_muF_ratio = muRrat/muFrat;
  return;
}



string PName[] = { "A-soft_2", "psi-soft_2", "N-soft" };



int main (int argc, char* argv[]) {


  /*************  input parameters  **************/

  // pole masses (GeV)
  mtop = 172.5;

  // relative precision for integrals
  INTEGRAL_PRECISION = 5.e-3;

  // scheme at small x (set to "false" for Q0MSbar scheme)
  MSbar = true;

  /***********************************************/


  // default values (modified by runtime arguments)
  double mH = 125;
  double sqrts = 8000;
  string PDFset = "NNPDF23_nnlo_as_0118.LHgrid";


  // read runtime arguments
  bool nob=true, noc=true; // when true, bottom and charm do NOT run into the loop coupling to the Higgs
  read_arguments(argc, argv, mH, sqrts, PDFset, nob, noc);


  // Start time counter
  struct timeval t0, t1;
  gettimeofday(&t0,NULL);


  // set to false to show intermediate K-factor results
  quiet = true;
  //
  // init PDFs
  cout << "Loading PDF set " << PDFset << endl;
  int subset = 0;
  pdf_init(PDFset, subset);
  //
  // masses (negative mass means the quark will not contribute to the loop)
  mbot = (nob ? -1 : 1) * LHAPDF::getQMass(5);
  mcha = (noc ? -1 : 1) * LHAPDF::getQMass(4);
  //
  // init
  double tau = mH*mH/sqrts/sqrts;
  sxc = new SmallxCoeffs("../ggHiggs/src/MBDFV.dat");
  init_consts(mH);
  SetScaleDependence(1,1);
  //
  cout << "Higgs mass:       " << mH << " GeV" << endl
       << "Collider energy:  " << sqrts << " GeV" << endl
       << "tau = mH^2/s =    " << tau << endl
       << "top    mass =     " << mtop << "  running in the loop" << endl
       << "bottom mass =     " << fabs(mbot) << "  " << (nob ? "NOT " : "") << "running in the loop" << endl
       << "charm  mass =     " << fabs(mcha) << "  " << (noc ? "NOT " : "") << "running in the loop" << endl
       << "quark mass effects beyond LO: " << (finite_mt ? "YES (exact theory)" : "NO (EFT)") << endl
       << "Renormalization scale muR = " << _muR_muF_ratio*_muF_mu_ratio << " * mH" << endl
       << "Factorization   scale muF = " << _muF_mu_ratio << " * mH" << endl
       << "Resummation prescription:   " << PName[prescr] << endl;
  //


  // Note also that the same PDF set has to be used at every order.
  double xsLO=1, xsNLO=0, xsNNLO=0, xsNNNLO=0;
  //
  double LOpref = xs_LO_prefactor(mH);
  double muF = mH * muFrat;
  double muR = mH * muRrat;
  double alpha_s = alphas(muR);
  // results
  double rexact[2], rsoft2, rhe;

  // LO
  xsLO = tau*Lum(tau,muF)*LOpref*pow(alpha_s,2);

  // NLO
  order = 1;
  __all_channels__ = true;
  compute(mH, sqrts, xsLO, 0, 0, &rexact[0]);
  xsNLO = xsLO + rexact[0];
  
  // NNLO
  order = 2;
  __all_channels__ = true;
  compute(mH, sqrts, xsLO, xsNLO, 0, &rexact[1]);
  xsNNLO = xsNLO + rexact[1];

  // NNNLO
  order = 3;
  __all_channels__ = false;
  compute(mH, sqrts, xsLO, xsNLO, xsNNLO, NULL, &rsoft2, NULL, NULL, &rhe);
  xsNNNLO = xsNNLO + rsoft2 + rhe;


  // *** Resummation part *** //
  //
  using namespace resumm;
  QCDResummation res(ggH, LHC, PDFset, subset, SILENT);
  //
  res.SetMtop(mtop);
  res.SetMbottom(mbot);
  res.SetMcharm(mcha);
  res.UseLargeMtopEFT(!finite_mt);
  res.SetLogCounting(primed);
  //
  // Now we select the prescription
  // The easiest way uses the name of the prescription, that can be
  //    "A-soft_n"   or   "psi-soft_n"   or   "N-soft"
  // where n is the order of the Altarelli-Parisi improvement, and can be 0, 1 or 2
  res.SetPrescriptionByName (PName[prescr], g0exp);
  //
  // Alternatively, one can set by hand all the parameters (useful for non-default prescriptions)
  /*
  // setup: default A-soft_2
  bool expBP = true;
  bool resDconst = true;
  APtype ap = AP2;
  prescription p = BP2;
  //
  // if psi-soft_2
  if(prescr == 1) {
    p = MP2;
    expBP = false;
    resDconst = false;
  }
  // if N-soft
  if(prescr == 2) {
    p = MP00;
    ap = AP0;
    expBP = false;
    resDconst = false;
    g0exp = false;
  }
  //
  res.SetAPimprovement(ap);
  res.SetPrescription (p, g0exp, resDconst, expBP);
  res.SetBPpars(10, 2);
  //
  */
  // compute resummed contribution
  double xsNLL    = xsLO * res.xsNLL  (mH, sqrts, muFrat, muRrat,   NLO);
  double xsNNLL   = xsLO * res.xsNNLL (mH, sqrts, muFrat, muRrat,  NNLO);
  double xsNNNLL2 = xsLO * res.xsNNNLL(mH, sqrts, muFrat, muRrat,  NNLO);
  double xsNNNLL3 = xsLO * res.xsNNNLL(mH, sqrts, muFrat, muRrat, NNNLO);
  //
  // end resummation part //



  cout << endl
       << "The following cross-sections (in pb) have been computed with the same PDF set (" << PDFset << ") at each order."
       << endl << endl;
  cout << "LO:         " << xsLO << endl;
  cout << "NLO:        " << xsNLO << endl;
  cout << "NLO+NLL':   " << xsNLO+xsNLL << endl;
  cout << "NNLO:       " << xsNNLO << endl;
  cout << "NNLO+NNLL': " << xsNNLO+xsNNLL << endl;
  cout << "NNLO+N3LL': " << xsNNLO+xsNNNLL2 << endl;
  cout << "N3LO:       " << xsNNNLO << endl;
  cout << "N3LO+N3LL': " << xsNNNLO+xsNNNLL3 << endl;
  //





  // Stop time counter
  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  cout << endl << "Total time: " << t << "s" << endl;


  return 0;

}
